class MTProtoProxyGenerator {
    constructor() {}

    // Convert a hex string to a byte array
    hexToBytes(hex) {
        const bytes = [];
        for (let c = 0; c < hex.length; c += 2) {
            bytes.push(parseInt(hex.substr(c, 2), 16));
        }
        return bytes;
    }

    // Convert a byte array to a hex string
    bytesToHex(bytes) {
        const hex = [];
        for (let i = 0; i < bytes.length; i++) {
            const current = bytes[i] < 0 ? bytes[i] + 256 : bytes[i];
            hex.push((current >>> 4).toString(16));
            hex.push((current & 0xF).toString(16));
        }
        return hex.join("");
    }

    // Convert a byte array to base64 string
    bytesToBase64(data) {
        const str = String.fromCharCode.apply(null, data);
        return Buffer.from(str, 'binary').toString('base64');
    }

    base64ToBytes(str) {
        return Buffer.from(str, 'base64');
    }

    // Convert a byte array to urlsafe base64 string
    bytesToUrlSafeBase64(data) {
        return this.bytesToBase64(data).replace(/\+/g, "-").replace(/\//g, "_");
    }

    urlSafeBase64ToBytes(str) {
        return this.base64ToBytes(str.replace(/-/g, "+").replace(/_/g, "/"));
    }

    // Parse query string to object
    parseQuery(queryString) {
        const query = {};
        const pairs = queryString.split('&');
        for (const pair of pairs) {
            const [key, value] = pair.split('=');
            query[decodeURIComponent(key)] = decodeURIComponent(value || '');
        }
        return query;
    }

    // Random integer generator
    getRandomInt(max) {
        return Math.floor(Math.random() * Math.floor(max));
    }

    // Generate a random hex secret
    generateRandomSecret() {
        const randHex = [];
        for (let i = 0; i < 16; i++) {
            randHex.push(this.getRandomInt(256));
        }
        return this.bytesToHex(randHex);
    }

    // Parse a telegram proxy URL
    parse(url) {
        const [proto, rest] = url.split("://");
        if (!(proto === "tg" || proto === "https")) {
            throw new Error(`Invalid URL protocol: ${proto}`);
        }

        const [host_or_path, queryString] = rest.split("?");
        if (proto === "tg" && host_or_path !== "proxy") {
            throw new Error(`Invalid path: ${host_or_path}`);
        } else if (proto === "https" && host_or_path !== "t.me/proxy") {
            throw new Error(`Invalid path: ${host_or_path}`);
        }

        const query = this.parseQuery(queryString);
        const { server, port, secret } = query;
        if (!server || !port || !secret) {
            throw new Error("Missing required fields: server, port, secret");
        }

        let hexSecret, protocol, tlsDomain = "";
        if (secret.length === 32 && /^[0-9a-f]+$/.test(secret)) {
            protocol = "Normal";
            hexSecret = secret;
        } else if (secret.length === 34 && secret.startsWith("dd") && /^[0-9a-f]+$/.test(secret)) {
            protocol = "Secure";
            hexSecret = secret.slice(-32);
        } else if (secret.length > 34 && secret.startsWith("ee") && /^[0-9a-f]+$/.test(secret)) {
            protocol = "Fake-TLS, hex secret";
            hexSecret = secret.slice(2, 34);
            tlsDomain = String.fromCharCode.apply(null, this.hexToBytes(secret.slice(34)));
        } else if (secret.startsWith("7")) {
            protocol = "Fake-TLS";
            let binSecret;
            if (/^[0-9a-zA-Z_=-]+$/.test(secret)) {
                binSecret = this.urlSafeBase64ToBytes(secret);
                protocol += ", URL-safe base64 secret";
            } else if (/^[0-9a-zA-Z\+\/=]+$/.test(secret)) {
                binSecret = this.base64ToBytes(secret);
                protocol += ", base64 secret";
            } else {
                throw new Error(`Invalid secret: ${secret}`);
            }
            hexSecret = this.bytesToHex(
                Array.from(binSecret.slice(1, 17), c => c.charCodeAt(0))
            );
            tlsDomain = binSecret.slice(17).toString('utf-8');
        } else {
            throw new Error(`Invalid secret: ${secret}`);
        }

        return { protocol, server, port, secret: hexSecret, domain: tlsDomain };
    }

    // Generate telegram proxy URLs
    generateUrls(server, port, secret, tlsDomain, urlType = 'tg://') {
        const secretBytes = this.hexToBytes(secret);
        const ddSecret = "dd" + secret;
        const domainBytes = Array.from(tlsDomain).map((c) => c.charCodeAt(0));
        const tlsBytes = this.hexToBytes("ee" + secret).concat(domainBytes);
        const tgPrefix = `${urlType}proxy?server=${server}&port=${port}&secret=`;

        return {
            normal: tgPrefix + secret,
            secure: tgPrefix + ddSecret,
            fakeTlsHex: tgPrefix + encodeURIComponent(this.bytesToHex(tlsBytes)),
            fakeTlsBase64: tgPrefix + this.bytesToBase64(tlsBytes),
            fakeTlsSafeBase64: tgPrefix + this.bytesToUrlSafeBase64(tlsBytes),
        };
    }
}

module.exports = MTProtoProxyGenerator;
